; CS-280 InterruptDemo.asm
; Purpose: Interrupt demo
; Note: use a tab count of 6 for decent formatting

.NOLIST
.INCLUDE "m32def.inc"		; contains .EQU's for DDRB, PORTB, and a lot more
.LIST

.EQU start = 0x2a			; define a symbol to represent address 0x2a in program memory
.CSEG						; this means further directives are for the Code Segment
.ORG 0x0
	rjmp 	start			; initialize Reset Vector at 0x0000

.CSEG						; switch back to Code Segment
.ORG start					; defines where these instructions start getting placed in Program Memory

; Initialize the Stack Pointer (SP) CPU register for subsequent use
.DEF temp=r30					; temp: register for temporary data store
	ldi		temp, LOW(RAMEND)	; get low byte of the address of the highest byte in Data Memory we can use
	out		SPL, temp			; load it into the low byte of SP
	ldi		temp, HIGH(RAMEND)	; high byte of address of highest Data Memory byte
	out		SPH, temp			; load it into the high byte of SP

	rcall	initPortB			; setup PortB for digital output
	rcall	setupInterrupts		; configure interrupts

	clr		temp				; reset temp variable
	
loop:
	out		PORTB, temp			; display value of temp on LEDs
	rcall 	short_delay			; wait a little
	inc 	r30					; increment temp
	rjmp 	loop				; and repeat forever

; interrupt handler int0_handler
; Purpose: implements the handling of the int0 external interrupt
int0_handler:
	; Note that I bit of SREG is CLEARED, preventing further interrupts until this routine returns!
	push	temp
	ldi		temp, 0x00
	out		PORTB, temp
	rcall	delay_500ms
	pop		temp
	reti					; return from interrupt (differs from ret in that it resets the I bit of SREG)
; end interrupt handler int0_handler

; subroutine initPortB
; Purpose: configure PORTB for digital output
initPortB:
	push	temp			; save
	ldi		temp, 0xff  	; set all bits to 1
	out 	DDRB, temp		; tell DDRB to configure pins as output (0xff means all output)
	out		PORTB, temp		; turn off all LEDs
	pop 	temp			; restore
	ret	
; end subroutine initPortB

; subroutine setupInterrupts
; enable global interrupt handling, and initilize interrupt handling specifically for INT0
setupInterrupts:
	push	temp				; save
	in		temp, GICR			; get current state of GICR (General Interrupt Control Register)
	sbr		temp, 0b01000000	; bits in GICR enables external interrupt 0
	out		GICR, temp			; set the bits

  ; MCUCR - MCU Control Register
	in		temp, MCUCR			; get current state of MCUCR
	sbr		temp, 0b00000010	; these bits in MCUCR define interrupts to occur on fall of signal on INT0 pin
	out		MCUCR, temp			; set the bits

	sei							; this ENABLES interrupts globally

	pop		temp				; restore
	ret
; end subroutine setupInterrupts
	
; subroutine short_delay
; Purpose: implement a delay of about 50ms with a clock speed of 16MHz
; Note: Works by calling the Delay subroutine with appropriate values for 
; the 'parameters' R0, R1, and R2
short_delay:
	; save the local registers we're using here
	push	R0
	push	R1
	push	R2
	push	temp

	ldi		temp, 20		; the following three values result in a 50ms delay
	mov		r0,	temp
	ldi		temp, 100
	mov		r1,	temp
	ldi		temp, 133
	mov		r2,	temp

	rcall	Delay

	pop		temp
	pop		R2
	pop		R1
	pop		R0
	ret
; end subroutine short_delay

/////////////////////////////////////////////////////////////////
; subroutine delay_500ms
; Purpose: implement a delay of about 500ms with a clock speed of 16MHz
; Note: Works by calling the Delay subroutine with appropriate values for 
; the 'parameters' R0, R1, and R2
delay_500ms:
	; save the local registers we're using here
	push	R0
	push	R1
	push	R2
	push	temp

	ldi		temp, 199		; the following three values result in a 500ms delay
	mov		r0,	temp
	ldi		temp, 100
	mov		r1,	temp
	ldi		temp, 133
	mov		r2,	temp

	rcall	Delay

	pop		temp
	pop		R2
	pop		R1
	pop		R0
	ret
; end subroutine delay_500ms
//////////////////////////////////////////////////////////////////

.include "delay.asm"
